@@ -9,11 +9,11 @@
  * and implementing search-path-controlled searches.
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/catalog/namespace.c,v 1.38.2.3 2007/04/20 02:38:58 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/catalog/namespace.c,v 1.58.2.1 2007/04/20 02:38:46 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -30,6 +30,7 @@
 #include "catalog/pg_proc.h"
 #include "catalog/pg_shadow.h"
 #include "catalog/pg_type.h"
+#include "commands/dbcommands.h"
 #include "lib/stringinfo.h"
 #include "miscadmin.h"
 #include "nodes/makefuncs.h"
@@ -159,6 +160,7 @@ Datum		pg_type_is_visible(PG_FUNCTION_ARGS);
 Datum		pg_function_is_visible(PG_FUNCTION_ARGS);
 Datum		pg_operator_is_visible(PG_FUNCTION_ARGS);
 Datum		pg_opclass_is_visible(PG_FUNCTION_ARGS);
+Datum		pg_conversion_is_visible(PG_FUNCTION_ARGS);
 
 
 /*
@@ -180,8 +182,10 @@ RangeVarGetRelid(const RangeVar *relation, bool failOK)
 	 */
 	if (relation->catalogname)
 	{
-		if (strcmp(relation->catalogname, DatabaseName) != 0)
-			elog(ERROR, "Cross-database references are not implemented");
+		if (strcmp(relation->catalogname, get_database_name(MyDatabaseId)) != 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+			   errmsg("cross-database references are not implemented")));
 	}
 
 	if (relation->schemaname)
@@ -199,11 +203,15 @@ RangeVarGetRelid(const RangeVar *relation, bool failOK)
 	if (!OidIsValid(relId) && !failOK)
 	{
 		if (relation->schemaname)
-			elog(ERROR, "Relation \"%s\".\"%s\" does not exist",
-				 relation->schemaname, relation->relname);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_TABLE),
+					 errmsg("relation \"%s.%s\" does not exist",
+							relation->schemaname, relation->relname)));
 		else
-			elog(ERROR, "Relation \"%s\" does not exist",
-				 relation->relname);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_TABLE),
+					 errmsg("relation \"%s\" does not exist",
+							relation->relname)));
 	}
 	return relId;
 }
@@ -227,15 +235,19 @@ RangeVarGetCreationNamespace(const RangeVar *newRelation)
 	 */
 	if (newRelation->catalogname)
 	{
-		if (strcmp(newRelation->catalogname, DatabaseName) != 0)
-			elog(ERROR, "Cross-database references are not implemented");
+		if (strcmp(newRelation->catalogname, get_database_name(MyDatabaseId)) != 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+			   errmsg("cross-database references are not implemented")));
 	}
 
 	if (newRelation->istemp)
 	{
 		/* TEMP tables are created in our backend-local temp namespace */
 		if (newRelation->schemaname)
-			elog(ERROR, "TEMP tables may not specify a namespace");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_TABLE_DEFINITION),
+				   errmsg("temporary tables may not specify a schema name")));
 		/* Initialize temp namespace if first time through */
 		if (!OidIsValid(myTempNamespace))
 			InitTempTableNamespace();
@@ -257,8 +269,10 @@ RangeVarGetCreationNamespace(const RangeVar *newRelation)
 								CStringGetDatum(newRelation->schemaname),
 									 0, 0, 0);
 		if (!OidIsValid(namespaceId))
-			elog(ERROR, "Namespace \"%s\" does not exist",
-				 newRelation->schemaname);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_SCHEMA),
+					 errmsg("schema \"%s\" does not exist",
+							newRelation->schemaname)));
 		/* we do not check for USAGE rights here! */
 	}
 	else
@@ -273,7 +287,9 @@ RangeVarGetCreationNamespace(const RangeVar *newRelation)
 		}
 		namespaceId = defaultCreationNamespace;
 		if (!OidIsValid(namespaceId))
-			elog(ERROR, "No namespace has been selected to create in");
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_SCHEMA),
+					 errmsg("no schema has been selected to create in")));
 	}
 
 	/* Note: callers will check for CREATE rights when appropriate */
@@ -296,7 +312,7 @@ RelnameGetRelid(const char *relname)
 
 	foreach(lptr, namespaceSearchPath)
 	{
-		Oid			namespaceId = (Oid) lfirsti(lptr);
+		Oid			namespaceId = lfirsto(lptr);
 
 		relid = get_relname_relid(relname, namespaceId);
 		if (OidIsValid(relid))
@@ -307,6 +323,7 @@ RelnameGetRelid(const char *relname)
 	return InvalidOid;
 }
 
+
 /*
  * RelationIsVisible
  *		Determine whether a relation (identified by OID) is visible in the
@@ -325,7 +342,7 @@ RelationIsVisible(Oid relid)
 							ObjectIdGetDatum(relid),
 							0, 0, 0);
 	if (!HeapTupleIsValid(reltup))
-		elog(ERROR, "Cache lookup failed for relation %u", relid);
+		elog(ERROR, "cache lookup failed for relation %u", relid);
 	relform = (Form_pg_class) GETSTRUCT(reltup);
 
 	recomputeNamespacePath();
@@ -333,11 +350,11 @@ RelationIsVisible(Oid relid)
 	/*
 	 * Quick check: if it ain't in the path at all, it ain't visible.
 	 * Items in the system namespace are surely in the path and so we
-	 * needn't even do intMember() for them.
+	 * needn't even do oidMember() for them.
 	 */
 	relnamespace = relform->relnamespace;
 	if (relnamespace != PG_CATALOG_NAMESPACE &&
-		!intMember(relnamespace, namespaceSearchPath))
+		!oidMember(relnamespace, namespaceSearchPath))
 		visible = false;
 	else
 	{
@@ -375,7 +392,7 @@ TypenameGetTypid(const char *typname)
 
 	foreach(lptr, namespaceSearchPath)
 	{
-		Oid			namespaceId = (Oid) lfirsti(lptr);
+		Oid			namespaceId = lfirsto(lptr);
 
 		typid = GetSysCacheOid(TYPENAMENSP,
 							   PointerGetDatum(typname),
@@ -407,7 +424,7 @@ TypeIsVisible(Oid typid)
 							ObjectIdGetDatum(typid),
 							0, 0, 0);
 	if (!HeapTupleIsValid(typtup))
-		elog(ERROR, "Cache lookup failed for type %u", typid);
+		elog(ERROR, "cache lookup failed for type %u", typid);
 	typform = (Form_pg_type) GETSTRUCT(typtup);
 
 	recomputeNamespacePath();
@@ -415,11 +432,11 @@ TypeIsVisible(Oid typid)
 	/*
 	 * Quick check: if it ain't in the path at all, it ain't visible.
 	 * Items in the system namespace are surely in the path and so we
-	 * needn't even do intMember() for them.
+	 * needn't even do oidMember() for them.
 	 */
 	typnamespace = typform->typnamespace;
 	if (typnamespace != PG_CATALOG_NAMESPACE &&
-		!intMember(typnamespace, namespaceSearchPath))
+		!oidMember(typnamespace, namespaceSearchPath))
 		visible = false;
 	else
 	{
@@ -516,7 +533,7 @@ FuncnameGetCandidates(List *names, int nargs)
 
 			foreach(nsp, namespaceSearchPath)
 			{
-				if (procform->pronamespace == (Oid) lfirsti(nsp) &&
+				if (procform->pronamespace == lfirsto(nsp) &&
 					procform->pronamespace != myTempNamespace)
 					break;
 				pathpos++;
@@ -612,7 +629,7 @@ FunctionIsVisible(Oid funcid)
 							 ObjectIdGetDatum(funcid),
 							 0, 0, 0);
 	if (!HeapTupleIsValid(proctup))
-		elog(ERROR, "Cache lookup failed for procedure %u", funcid);
+		elog(ERROR, "cache lookup failed for function %u", funcid);
 	procform = (Form_pg_proc) GETSTRUCT(proctup);
 
 	recomputeNamespacePath();
@@ -620,11 +637,11 @@ FunctionIsVisible(Oid funcid)
 	/*
 	 * Quick check: if it ain't in the path at all, it ain't visible.
 	 * Items in the system namespace are surely in the path and so we
-	 * needn't even do intMember() for them.
+	 * needn't even do oidMember() for them.
 	 */
 	pronamespace = procform->pronamespace;
 	if (pronamespace != PG_CATALOG_NAMESPACE &&
-		!intMember(pronamespace, namespaceSearchPath))
+		!oidMember(pronamespace, namespaceSearchPath))
 		visible = false;
 	else
 	{
@@ -735,7 +752,7 @@ OpernameGetCandidates(List *names, char oprkind)
 
 			foreach(nsp, namespaceSearchPath)
 			{
-				if (operform->oprnamespace == (Oid) lfirsti(nsp) &&
+				if (operform->oprnamespace == lfirsto(nsp) &&
 					operform->oprnamespace != myTempNamespace)
 					break;
 				pathpos++;
@@ -828,7 +845,7 @@ OperatorIsVisible(Oid oprid)
 							ObjectIdGetDatum(oprid),
 							0, 0, 0);
 	if (!HeapTupleIsValid(oprtup))
-		elog(ERROR, "Cache lookup failed for operator %u", oprid);
+		elog(ERROR, "cache lookup failed for operator %u", oprid);
 	oprform = (Form_pg_operator) GETSTRUCT(oprtup);
 
 	recomputeNamespacePath();
@@ -836,11 +853,11 @@ OperatorIsVisible(Oid oprid)
 	/*
 	 * Quick check: if it ain't in the path at all, it ain't visible.
 	 * Items in the system namespace are surely in the path and so we
-	 * needn't even do intMember() for them.
+	 * needn't even do oidMember() for them.
 	 */
 	oprnamespace = oprform->oprnamespace;
 	if (oprnamespace != PG_CATALOG_NAMESPACE &&
-		!intMember(oprnamespace, namespaceSearchPath))
+		!oidMember(oprnamespace, namespaceSearchPath))
 		visible = false;
 	else
 	{
@@ -912,7 +929,7 @@ OpclassGetCandidates(Oid amid)
 		/* Consider only opclasses that are in the search path */
 		foreach(nsp, namespaceSearchPath)
 		{
-			if (opcform->opcnamespace == (Oid) lfirsti(nsp) &&
+			if (opcform->opcnamespace == lfirsto(nsp) &&
 				opcform->opcnamespace != myTempNamespace)
 				break;
 			pathpos++;
@@ -1009,7 +1026,7 @@ OpclassnameGetOpcid(Oid amid, const char *opcname)
 
 	foreach(lptr, namespaceSearchPath)
 	{
-		Oid			namespaceId = (Oid) lfirsti(lptr);
+		Oid			namespaceId = lfirsto(lptr);
 
 		if (namespaceId == myTempNamespace)
 			continue;			/* do not look in temp namespace */
@@ -1045,7 +1062,7 @@ OpclassIsVisible(Oid opcid)
 							ObjectIdGetDatum(opcid),
 							0, 0, 0);
 	if (!HeapTupleIsValid(opctup))
-		elog(ERROR, "Cache lookup failed for opclass %u", opcid);
+		elog(ERROR, "cache lookup failed for opclass %u", opcid);
 	opcform = (Form_pg_opclass) GETSTRUCT(opctup);
 
 	recomputeNamespacePath();
@@ -1053,11 +1070,11 @@ OpclassIsVisible(Oid opcid)
 	/*
 	 * Quick check: if it ain't in the path at all, it ain't visible.
 	 * Items in the system namespace are surely in the path and so we
-	 * needn't even do intMember() for them.
+	 * needn't even do oidMember() for them.
 	 */
 	opcnamespace = opcform->opcnamespace;
 	if (opcnamespace != PG_CATALOG_NAMESPACE &&
-		!intMember(opcnamespace, namespaceSearchPath))
+		!oidMember(opcnamespace, namespaceSearchPath))
 		visible = false;
 	else
 	{
@@ -1077,6 +1094,90 @@ OpclassIsVisible(Oid opcid)
 	return visible;
 }
 
+/*
+ * ConversionGetConid
+ *		Try to resolve an unqualified conversion name.
+ *		Returns OID if conversion found in search path, else InvalidOid.
+ *
+ * This is essentially the same as RelnameGetRelid.
+ */
+Oid
+ConversionGetConid(const char *conname)
+{
+	Oid			conid;
+	List	   *lptr;
+
+	recomputeNamespacePath();
+
+	foreach(lptr, namespaceSearchPath)
+	{
+		Oid			namespaceId = lfirsto(lptr);
+
+		if (namespaceId == myTempNamespace)
+			continue;			/* do not look in temp namespace */
+
+		conid = GetSysCacheOid(CONNAMENSP,
+							   PointerGetDatum(conname),
+							   ObjectIdGetDatum(namespaceId),
+							   0, 0);
+		if (OidIsValid(conid))
+			return conid;
+	}
+
+	/* Not found in path */
+	return InvalidOid;
+}
+
+/*
+ * ConversionIsVisible
+ *		Determine whether a conversion (identified by OID) is visible in the
+ *		current search path.  Visible means "would be found by searching
+ *		for the unqualified conversion name".
+ */
+bool
+ConversionIsVisible(Oid conid)
+{
+	HeapTuple	contup;
+	Form_pg_conversion conform;
+	Oid			connamespace;
+	bool		visible;
+
+	contup = SearchSysCache(CONOID,
+							ObjectIdGetDatum(conid),
+							0, 0, 0);
+	if (!HeapTupleIsValid(contup))
+		elog(ERROR, "cache lookup failed for conversion %u", conid);
+	conform = (Form_pg_conversion) GETSTRUCT(contup);
+
+	recomputeNamespacePath();
+
+	/*
+	 * Quick check: if it ain't in the path at all, it ain't visible.
+	 * Items in the system namespace are surely in the path and so we
+	 * needn't even do oidMember() for them.
+	 */
+	connamespace = conform->connamespace;
+	if (connamespace != PG_CATALOG_NAMESPACE &&
+		!oidMember(connamespace, namespaceSearchPath))
+		visible = false;
+	else
+	{
+		/*
+		 * If it is in the path, it might still not be visible; it could
+		 * be hidden by another conversion of the same name earlier in the
+		 * path. So we must do a slow check to see if this conversion
+		 * would be found by ConversionGetConid.
+		 */
+		char	   *conname = NameStr(conform->conname);
+
+		visible = (ConversionGetConid(conname) == conid);
+	}
+
+	ReleaseSysCache(contup);
+
+	return visible;
+}
+
 /*
  * DeconstructQualifiedName
  *		Given a possibly-qualified name expressed as a list of String nodes,
@@ -1105,17 +1206,21 @@ DeconstructQualifiedName(List *names,
 		case 3:
 			catalogname = strVal(lfirst(names));
 			schemaname = strVal(lsecond(names));
-			objname = strVal(lfirst(lnext(lnext(names))));
+			objname = strVal(lthird(names));
 
 			/*
 			 * We check the catalog name and then ignore it.
 			 */
-			if (strcmp(catalogname, DatabaseName) != 0)
-				elog(ERROR, "Cross-database references are not implemented");
+			if (strcmp(catalogname, get_database_name(MyDatabaseId)) != 0)
+				ereport(ERROR,
+						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				errmsg("cross-database references are not implemented")));
 			break;
 		default:
-			elog(ERROR, "Improper qualified name (too many dotted names): %s",
-				 NameListToString(names));
+			ereport(ERROR,
+					(errcode(ERRCODE_SYNTAX_ERROR),
+			errmsg("improper qualified name (too many dotted names): %s",
+				   NameListToString(names))));
 			break;
 	}
 
@@ -1128,7 +1233,7 @@ DeconstructQualifiedName(List *names,
  *		Process an explicitly-specified schema name: look up the schema
  *		and verify we have USAGE (lookup) rights in it.
  *
- * Returns the namespace OID.  Raises elog if any problem.
+ * Returns the namespace OID.  Raises ereport if any problem.
  */
 Oid
 LookupExplicitNamespace(const char *nspname)
@@ -1153,11 +1258,14 @@ LookupExplicitNamespace(const char *nspname)
 								 CStringGetDatum(nspname),
 								 0, 0, 0);
 	if (!OidIsValid(namespaceId))
-		elog(ERROR, "Namespace \"%s\" does not exist", nspname);
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_SCHEMA),
+				 errmsg("schema \"%s\" does not exist", nspname)));
 
 	aclresult = pg_namespace_aclcheck(namespaceId, GetUserId(), ACL_USAGE);
 	if (aclresult != ACLCHECK_OK)
-		aclcheck_error(aclresult, nspname);
+		aclcheck_error(aclresult, ACL_KIND_NAMESPACE,
+					   nspname);
 
 	return namespaceId;
 }
@@ -1195,8 +1303,9 @@ QualifiedNameGetCreationNamespace(List *names, char **objname_p)
 									 CStringGetDatum(schemaname),
 									 0, 0, 0);
 		if (!OidIsValid(namespaceId))
-			elog(ERROR, "Namespace \"%s\" does not exist",
-				 schemaname);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_SCHEMA),
+					 errmsg("schema \"%s\" does not exist", schemaname)));
 		/* we do not check for USAGE rights here! */
 	}
 	else
@@ -1211,7 +1320,9 @@ QualifiedNameGetCreationNamespace(List *names, char **objname_p)
 		}
 		namespaceId = defaultCreationNamespace;
 		if (!OidIsValid(namespaceId))
-			elog(ERROR, "No namespace has been selected to create in");
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_SCHEMA),
+					 errmsg("no schema has been selected to create in")));
 	}
 
 	/* Note: callers will check for CREATE rights when appropriate */
@@ -1240,10 +1351,13 @@ makeRangeVarFromNameList(List *names)
 		case 3:
 			rel->catalogname = strVal(lfirst(names));
 			rel->schemaname = strVal(lsecond(names));
-			rel->relname = strVal(lfirst(lnext(lnext(names))));
+			rel->relname = strVal(lthird(names));
 			break;
 		default:
-			elog(ERROR, "Improper relation name (too many dotted names)");
+			ereport(ERROR,
+					(errcode(ERRCODE_SYNTAX_ERROR),
+			 errmsg("improper relation name (too many dotted names): %s",
+					NameListToString(names))));
 			break;
 	}
 
@@ -1269,7 +1383,7 @@ NameListToString(List *names)
 	{
 		if (l != names)
 			appendStringInfoChar(&string, '.');
-		appendStringInfo(&string, "%s", strVal(lfirst(l)));
+		appendStringInfoString(&string, strVal(lfirst(l)));
 	}
 
 	return string.data;
@@ -1294,7 +1408,7 @@ NameListToQuotedString(List *names)
 	{
 		if (l != names)
 			appendStringInfoChar(&string, '.');
-		appendStringInfo(&string, "%s", quote_identifier(strVal(lfirst(l))));
+		appendStringInfoString(&string, quote_identifier(strVal(lfirst(l))));
 	}
 
 	return string.data;
@@ -1391,7 +1505,7 @@ FindConversionByName(List *name)
 
 		foreach(lptr, namespaceSearchPath)
 		{
-			namespaceId = (Oid) lfirsti(lptr);
+			namespaceId = lfirsto(lptr);
 
 			if (namespaceId == myTempNamespace)
 				continue;			/* do not look in temp namespace */
@@ -1419,7 +1533,7 @@ FindDefaultConversionProc(int4 for_encoding, int4 to_encoding)
 
 	foreach(lptr, namespaceSearchPath)
 	{
-		Oid			namespaceId = (Oid) lfirsti(lptr);
+		Oid			namespaceId = lfirsto(lptr);
 
 		if (namespaceId == myTempNamespace)
 			continue;			/* do not look in temp namespace */
@@ -1439,7 +1553,7 @@ FindDefaultConversionProc(int4 for_encoding, int4 to_encoding)
 static void
 recomputeNamespacePath(void)
 {
-	Oid			userId = GetUserId();
+	AclId		userId = GetUserId();
 	char	   *rawname;
 	List	   *namelist;
 	List	   *oidlist;
@@ -1463,7 +1577,7 @@ recomputeNamespacePath(void)
 	{
 		/* syntax error in name list */
 		/* this should not happen if GUC checked check_search_path */
-		elog(ERROR, "recomputeNamespacePath: invalid list syntax");
+		elog(ERROR, "invalid list syntax");
 	}
 
 	/*
@@ -1497,10 +1611,10 @@ recomputeNamespacePath(void)
 											 0, 0, 0);
 				ReleaseSysCache(tuple);
 				if (OidIsValid(namespaceId) &&
-					!intMember(namespaceId, oidlist) &&
+					!oidMember(namespaceId, oidlist) &&
 					pg_namespace_aclcheck(namespaceId, userId,
 										  ACL_USAGE) == ACLCHECK_OK)
-					oidlist = lappendi(oidlist, namespaceId);
+					oidlist = lappendo(oidlist, namespaceId);
 			}
 		}
 		else if (strcmp(curname, "pg_temp") == 0)
@@ -1508,8 +1622,8 @@ recomputeNamespacePath(void)
 			/* pg_temp --- substitute temp namespace, if any */
 			if (OidIsValid(myTempNamespace))
 			{
-				if (!intMember(myTempNamespace, oidlist))
-					oidlist = lappendi(oidlist, myTempNamespace);
+				if (!oidMember(myTempNamespace, oidlist))
+					oidlist = lappendo(oidlist, myTempNamespace);
 			}
 			else
 			{
@@ -1525,10 +1639,10 @@ recomputeNamespacePath(void)
 										 CStringGetDatum(curname),
 										 0, 0, 0);
 			if (OidIsValid(namespaceId) &&
-				!intMember(namespaceId, oidlist) &&
+				!oidMember(namespaceId, oidlist) &&
 				pg_namespace_aclcheck(namespaceId, userId,
 									  ACL_USAGE) == ACLCHECK_OK)
-				oidlist = lappendi(oidlist, namespaceId);
+				oidlist = lappendo(oidlist, namespaceId);
 		}
 	}
 
@@ -1540,23 +1654,23 @@ recomputeNamespacePath(void)
 	if (oidlist == NIL)
 		firstNS = InvalidOid;
 	else
-		firstNS = (Oid) lfirsti(oidlist);
+		firstNS = lfirsto(oidlist);
 
 	/*
 	 * Add any implicitly-searched namespaces to the list.	Note these go
 	 * on the front, not the back; also notice that we do not check USAGE
 	 * permissions for these.
 	 */
-	if (!intMember(PG_CATALOG_NAMESPACE, oidlist))
-		oidlist = lconsi(PG_CATALOG_NAMESPACE, oidlist);
+	if (!oidMember(PG_CATALOG_NAMESPACE, oidlist))
+		oidlist = lconso(PG_CATALOG_NAMESPACE, oidlist);
 
 	if (OidIsValid(myTempNamespace) &&
-		!intMember(myTempNamespace, oidlist))
-		oidlist = lconsi(myTempNamespace, oidlist);
+		!oidMember(myTempNamespace, oidlist))
+		oidlist = lconso(myTempNamespace, oidlist);
 
 	if (OidIsValid(mySpecialNamespace) &&
-		!intMember(mySpecialNamespace, oidlist))
-		oidlist = lconsi(mySpecialNamespace, oidlist);
+		!oidMember(mySpecialNamespace, oidlist))
+		oidlist = lconso(mySpecialNamespace, oidlist);
 
 	/*
 	 * Now that we've successfully built the new list of namespace OIDs,
@@ -1619,10 +1733,12 @@ InitTempTableNamespace(void)
 	 */
 	if (pg_database_aclcheck(MyDatabaseId, GetSessionUserId(),
 							 ACL_CREATE_TEMP) != ACLCHECK_OK)
-		elog(ERROR, "%s: not authorized to create temp tables",
-			 DatabaseName);
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+				 errmsg("permission denied to create temporary tables in database \"%s\"",
+						get_database_name(MyDatabaseId))));
 
-	snprintf(namespaceName, NAMEDATALEN, "pg_temp_%d", MyBackendId);
+	snprintf(namespaceName, sizeof(namespaceName), "pg_temp_%d", MyBackendId);
 
 	namespaceId = GetSysCacheOid(NAMESPACENAME,
 								 CStringGetDatum(namespaceName),
@@ -1714,8 +1830,8 @@ RemoveTempRelations(Oid tempNamespaceId)
 
 	/*
 	 * We want to get rid of everything in the target namespace, but not
-	 * the namespace itself (deleting it only to recreate it later would be
-	 * a waste of cycles).  We do this by finding everything that has a
+	 * the namespace itself (deleting it only to recreate it later would
+	 * be a waste of cycles).  We do this by finding everything that has a
 	 * dependency on the namespace.
 	 */
 	object.classId = get_system_catalog_relid(NamespaceRelationName);
@@ -1735,11 +1851,11 @@ RemoveTempRelationsCallback(void)
 	{
 		/* Need to ensure we have a usable transaction. */
 		AbortOutOfAnyTransaction();
-		StartTransactionCommand(true);
+		StartTransactionCommand();
 
 		RemoveTempRelations(myTempNamespace);
 
-		CommitTransactionCommand(true);
+		CommitTransactionCommand();
 	}
 }
 
@@ -1791,7 +1907,9 @@ assign_search_path(const char *newval, bool doit, bool interactive)
 			if (!SearchSysCacheExists(NAMESPACENAME,
 									  CStringGetDatum(curname),
 									  0, 0, 0))
-				elog(ERROR, "Namespace \"%s\" does not exist", curname);
+				ereport(ERROR,
+						(errcode(ERRCODE_UNDEFINED_SCHEMA),
+					   errmsg("schema \"%s\" does not exist", curname)));
 		}
 	}
 
@@ -1827,7 +1945,7 @@ InitializeSearchPath(void)
 		MemoryContext oldcxt;
 
 		oldcxt = MemoryContextSwitchTo(TopMemoryContext);
-		namespaceSearchPath = makeListi1(PG_CATALOG_NAMESPACE);
+		namespaceSearchPath = makeListo1(PG_CATALOG_NAMESPACE);
 		MemoryContextSwitchTo(oldcxt);
 		defaultCreationNamespace = PG_CATALOG_NAMESPACE;
 		firstExplicitNamespace = PG_CATALOG_NAMESPACE;
@@ -1894,7 +2012,7 @@ fetch_search_path(bool includeImplicit)
 	result = namespaceSearchPath;
 	if (!includeImplicit)
 	{
-		while (result && (Oid) lfirsti(result) != firstExplicitNamespace)
+		while (result && lfirsto(result) != firstExplicitNamespace)
 			result = lnext(result);
 	}
 
@@ -1944,3 +2062,11 @@ pg_opclass_is_visible(PG_FUNCTION_ARGS)
 
 	PG_RETURN_BOOL(OpclassIsVisible(oid));
 }
+
+Datum
+pg_conversion_is_visible(PG_FUNCTION_ARGS)
+{
+	Oid			oid = PG_GETARG_OID(0);
+
+	PG_RETURN_BOOL(ConversionIsVisible(oid));
+}
