@@ -3,27 +3,29 @@
  * float.c
  *	  Functions for the built-in floating-point types.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/float.c,v 1.70 2001/03/22 03:59:50 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/float.c,v 1.94 2003/09/25 06:58:03 petere Exp $
  *
  *-------------------------------------------------------------------------
  */
-/*
+/*----------
  * OLD COMMENTS
  *		Basic float4 ops:
- *		 float4in, float4out, float4abs, float4um
+ *		 float4in, float4out, float4recv, float4send
+ *		 float4abs, float4um, float4up
  *		Basic float8 ops:
- *		 float8in, float8out, float8abs, float8um
+ *		 float8in, float8out, float8recv, float8send
+ *		 float8abs, float8um, float8up
  *		Arithmetic operators:
  *		 float4pl, float4mi, float4mul, float4div
  *		 float8pl, float8mi, float8mul, float8div
  *		Comparison operators:
- *		 float4eq, float4ne, float4lt, float4le, float4gt, float4ge
- *		 float8eq, float8ne, float8lt, float8le, float8gt, float8ge
+ *		 float4eq, float4ne, float4lt, float4le, float4gt, float4ge, float4cmp
+ *		 float8eq, float8ne, float8lt, float8le, float8gt, float8ge, float8cmp
  *		Conversion routines:
  *		 ftod, dtof, i4tod, dtoi4, i2tod, dtoi2, itof, ftoi, i2tof, ftoi2
  *
@@ -37,7 +39,8 @@
  *		 float84eq, float84ne, float84lt, float84le, float84gt, float84ge
  *
  *		(You can do the arithmetic and comparison stuff using conversion
- *		 routines, but then you pay the overhead of converting...)
+ *		 routines, but then you pay the overhead of invoking a separate
+ *		 conversion function...)
  *
  * XXX GLUESOME STUFF. FIX IT! -AY '94
  *
@@ -45,58 +48,31 @@
  *		 a bit of the existing code. Need to change the error checking
  *		 for calls to pow(), exp() since on some machines (my Linux box
  *		 included) these routines do not set errno. - tgl 97/05/10
+ *----------
  */
+#include "postgres.h"
+
 #include <ctype.h>
 #include <errno.h>
 #include <float.h>				/* faked on sunos4 */
 #include <math.h>
 
-#include "postgres.h"
-
 #include <limits.h>
 /* for finite() on Solaris */
 #ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
 #endif
 
+#include "catalog/pg_type.h"
 #include "fmgr.h"
+#include "libpq/pqformat.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
 
 
-#if !(NeXT && NX_CURRENT_COMPILER_RELEASE > NX_COMPILER_RELEASE_3_2)
- /* NS3.3 has conflicting declarations of these in <math.h> */
-
-#ifndef atof
-extern double atof(const char *p);
-
-#endif
-
 #ifndef HAVE_CBRT
-#define cbrt my_cbrt
 static double cbrt(double x);
-
-#else
-#if !defined(nextstep)
-extern double cbrt(double x);
-
-#endif
-#endif	 /* HAVE_CBRT */
-
-#ifndef HAVE_RINT
-#define rint my_rint
-static double rint(double x);
-
-#else
-extern double rint(double x);
-
-#endif	 /* HAVE_RINT */
-
-#endif	 /* NeXT check */
-
-
-static void CheckFloat4Val(double val);
-static void CheckFloat8Val(double val);
+#endif   /* HAVE_CBRT */
 
 #ifndef M_PI
 /* from my RH5.2 gcc math.h file - thomas 2000-04-03 */
@@ -114,8 +90,6 @@ static void CheckFloat8Val(double val);
 #define SHRT_MIN (-32768)
 #endif
 
-#define FORMAT			'g'		/* use "g" output format as standard
-								 * format */
 /* not sure what the following should be, but better to make it over-sufficient */
 #define MAXFLOATWIDTH	64
 #define MAXDOUBLEWIDTH	128
@@ -129,16 +103,25 @@ static void CheckFloat8Val(double val);
 #define FLOAT8_MIN		 DBL_MIN
 
 
-/*
-   check to see if a float4 val is outside of
-   the FLOAT4_MIN, FLOAT4_MAX bounds.
+/* Configurable GUC parameter */
+int			extra_float_digits = 0;		/* Added to DBL_DIG or FLT_DIG */
 
-   raise an elog warning if it is
+
+static void CheckFloat4Val(double val);
+static void CheckFloat8Val(double val);
+static int	float4_cmp_internal(float4 a, float4 b);
+static int	float8_cmp_internal(float8 a, float8 b);
+
+
+/*
+ * check to see if a float4 val is outside of
+ * the FLOAT4_MIN, FLOAT4_MAX bounds.
+ *
+ * raise an ereport warning if it is
 */
 static void
 CheckFloat4Val(double val)
 {
-
 	/*
 	 * defining unsafe floats's will make float4 and float8 ops faster at
 	 * the cost of safety, of course!
@@ -147,23 +130,27 @@ CheckFloat4Val(double val)
 	return;
 #else
 	if (fabs(val) > FLOAT4_MAX)
-		elog(ERROR, "Bad float4 input format -- overflow");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"real\" value out of range: overflow")));
 	if (val != 0.0 && fabs(val) < FLOAT4_MIN)
-		elog(ERROR, "Bad float4 input format -- underflow");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"real\" value out of range: underflow")));
+
 	return;
-#endif	 /* UNSAFE_FLOATS */
+#endif   /* UNSAFE_FLOATS */
 }
 
 /*
-   check to see if a float8 val is outside of
-   the FLOAT8_MIN, FLOAT8_MAX bounds.
-
-   raise an elog warning if it is
-*/
+ * check to see if a float8 val is outside of
+ * the FLOAT8_MIN, FLOAT8_MAX bounds.
+ *
+ * raise an ereport error if it is
+ */
 static void
 CheckFloat8Val(double val)
 {
-
 	/*
 	 * defining unsafe floats's will make float4 and float8 ops faster at
 	 * the cost of safety, of course!
@@ -172,11 +159,14 @@ CheckFloat8Val(double val)
 	return;
 #else
 	if (fabs(val) > FLOAT8_MAX)
-		elog(ERROR, "Bad float8 input format -- overflow");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"double precision\" value out of range: overflow")));
 	if (val != 0.0 && fabs(val) < FLOAT8_MIN)
-		elog(ERROR, "Bad float8 input format -- underflow");
-	return;
-#endif	 /* UNSAFE_FLOATS */
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"double precision\" value out of range: underflow")));
+#endif   /* UNSAFE_FLOATS */
 }
 
 /*
@@ -197,13 +187,25 @@ float4in(PG_FUNCTION_ARGS)
 	val = strtod(num, &endptr);
 	if (*endptr != '\0')
 	{
-		/* Should we accept "NaN" or "Infinity" for float4? */
-		elog(ERROR, "Bad float4 input format '%s'", num);
+		/*
+		 * XXX we should accept "Infinity" and "-Infinity" too, but what
+		 * are the correct values to assign?  HUGE_VAL will provoke an
+		 * error from CheckFloat4Val.
+		 */
+		if (strcasecmp(num, "NaN") == 0)
+			val = NAN;
+		else
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+					 errmsg("invalid input syntax for type real: \"%s\"",
+							num)));
 	}
 	else
 	{
 		if (errno == ERANGE)
-			elog(ERROR, "Input '%s' is out of range for float4", num);
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+					 errmsg("\"%s\" is out of range for type real", num)));
 	}
 
 	/*
@@ -224,11 +226,51 @@ float4out(PG_FUNCTION_ARGS)
 {
 	float4		num = PG_GETARG_FLOAT4(0);
 	char	   *ascii = (char *) palloc(MAXFLOATWIDTH + 1);
+	int			infflag;
+	int			ndig;
+
+	if (isnan(num))
+		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
+	infflag = isinf(num);
+	if (infflag > 0)
+		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
+	if (infflag < 0)
+		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
+
+	ndig = FLT_DIG + extra_float_digits;
+	if (ndig < 1)
+		ndig = 1;
+
+	sprintf(ascii, "%.*g", ndig, num);
 
-	sprintf(ascii, "%.*g", FLT_DIG, num);
 	PG_RETURN_CSTRING(ascii);
 }
 
+/*
+ *		float4recv			- converts external binary format to float4
+ */
+Datum
+float4recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_FLOAT4(pq_getmsgfloat4(buf));
+}
+
+/*
+ *		float4send			- converts float4 to binary format
+ */
+Datum
+float4send(PG_FUNCTION_ARGS)
+{
+	float4		num = PG_GETARG_FLOAT4(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendfloat4(&buf, num);
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
 /*
  *		float8in		- converts "num" to float8
  *						  restricted syntax:
@@ -251,13 +293,20 @@ float8in(PG_FUNCTION_ARGS)
 			val = NAN;
 		else if (strcasecmp(num, "Infinity") == 0)
 			val = HUGE_VAL;
+		else if (strcasecmp(num, "-Infinity") == 0)
+			val = -HUGE_VAL;
 		else
-			elog(ERROR, "Bad float8 input format '%s'", num);
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+					 errmsg("invalid input syntax for type double precision: \"%s\"",
+							num)));
 	}
 	else
 	{
 		if (errno == ERANGE)
-			elog(ERROR, "Input '%s' is out of range for float8", num);
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+					 errmsg("\"%s\" is out of range for type double precision", num)));
 	}
 
 	CheckFloat8Val(val);
@@ -265,7 +314,6 @@ float8in(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(val);
 }
 
-
 /*
  *		float8out		- converts float8 number to a string
  *						  using a standard output format
@@ -275,16 +323,52 @@ float8out(PG_FUNCTION_ARGS)
 {
 	float8		num = PG_GETARG_FLOAT8(0);
 	char	   *ascii = (char *) palloc(MAXDOUBLEWIDTH + 1);
+	int			infflag;
+	int			ndig;
 
 	if (isnan(num))
 		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
-	if (isinf(num))
+	infflag = isinf(num);
+	if (infflag > 0)
 		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
+	if (infflag < 0)
+		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
+
+	ndig = DBL_DIG + extra_float_digits;
+	if (ndig < 1)
+		ndig = 1;
+
+	sprintf(ascii, "%.*g", ndig, num);
 
-	sprintf(ascii, "%.*g", DBL_DIG, num);
 	PG_RETURN_CSTRING(ascii);
 }
 
+/*
+ *		float8recv			- converts external binary format to float8
+ */
+Datum
+float8recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_FLOAT8(pq_getmsgfloat8(buf));
+}
+
+/*
+ *		float8send			- converts float8 to binary format
+ */
+Datum
+float8send(PG_FUNCTION_ARGS)
+{
+	float8		num = PG_GETARG_FLOAT8(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendfloat8(&buf, num);
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
+
 /* ========== PUBLIC ROUTINES ========== */
 
 
@@ -316,6 +400,14 @@ float4um(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT4((float4) -arg1);
 }
 
+Datum
+float4up(PG_FUNCTION_ARGS)
+{
+	float4		arg = PG_GETARG_FLOAT4(0);
+
+	PG_RETURN_FLOAT4(arg);
+}
+
 Datum
 float4larger(PG_FUNCTION_ARGS)
 {
@@ -323,7 +415,10 @@ float4larger(PG_FUNCTION_ARGS)
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 	float4		result;
 
-	result = ((arg1 > arg2) ? arg1 : arg2);
+	if (float4_cmp_internal(arg1, arg2) > 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT4(result);
 }
 
@@ -334,7 +429,10 @@ float4smaller(PG_FUNCTION_ARGS)
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 	float4		result;
 
-	result = ((arg1 < arg2) ? arg1 : arg2);
+	if (float4_cmp_internal(arg1, arg2) < 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT4(result);
 }
 
@@ -375,6 +473,14 @@ float8um(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(result);
 }
 
+Datum
+float8up(PG_FUNCTION_ARGS)
+{
+	float8		arg = PG_GETARG_FLOAT8(0);
+
+	PG_RETURN_FLOAT8(arg);
+}
+
 Datum
 float8larger(PG_FUNCTION_ARGS)
 {
@@ -382,8 +488,10 @@ float8larger(PG_FUNCTION_ARGS)
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 	float8		result;
 
-	result = ((arg1 > arg2) ? arg1 : arg2);
-
+	if (float8_cmp_internal(arg1, arg2) > 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT8(result);
 }
 
@@ -394,8 +502,10 @@ float8smaller(PG_FUNCTION_ARGS)
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 	float8		result;
 
-	result = ((arg1 < arg2) ? arg1 : arg2);
-
+	if (float8_cmp_internal(arg1, arg2) < 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT8(result);
 }
 
@@ -456,7 +566,9 @@ float4div(PG_FUNCTION_ARGS)
 	double		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float4div: divide by zero error");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	/* Do division in float8, then check for overflow */
 	result = (float8) arg1 / (float8) arg2;
@@ -518,7 +630,9 @@ float8div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float8div: divide by zero error");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 
@@ -536,13 +650,43 @@ float8div(PG_FUNCTION_ARGS)
 /*
  *		float4{eq,ne,lt,le,gt,ge}		- float4/float4 comparison operations
  */
+static int
+float4_cmp_internal(float4 a, float4 b)
+{
+	/*
+	 * We consider all NANs to be equal and larger than any non-NAN. This
+	 * is somewhat arbitrary; the important thing is to have a consistent
+	 * sort order.
+	 */
+	if (isnan(a))
+	{
+		if (isnan(b))
+			return 0;			/* NAN = NAN */
+		else
+			return 1;			/* NAN > non-NAN */
+	}
+	else if (isnan(b))
+	{
+		return -1;				/* non-NAN < NAN */
+	}
+	else
+	{
+		if (a > b)
+			return 1;
+		else if (a < b)
+			return -1;
+		else
+			return 0;
+	}
+}
+
 Datum
 float4eq(PG_FUNCTION_ARGS)
 {
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 == arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) == 0);
 }
 
 Datum
@@ -551,7 +695,7 @@ float4ne(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 != arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) != 0);
 }
 
 Datum
@@ -560,7 +704,7 @@ float4lt(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 < arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) < 0);
 }
 
 Datum
@@ -569,7 +713,7 @@ float4le(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 <= arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) <= 0);
 }
 
 Datum
@@ -578,7 +722,7 @@ float4gt(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 > arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) > 0);
 }
 
 Datum
@@ -587,19 +731,58 @@ float4ge(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 >= arg2);
+	PG_RETURN_BOOL(float4_cmp_internal(arg1, arg2) >= 0);
+}
+
+Datum
+btfloat4cmp(PG_FUNCTION_ARGS)
+{
+	float4		arg1 = PG_GETARG_FLOAT4(0);
+	float4		arg2 = PG_GETARG_FLOAT4(1);
+
+	PG_RETURN_INT32(float4_cmp_internal(arg1, arg2));
 }
 
 /*
  *		float8{eq,ne,lt,le,gt,ge}		- float8/float8 comparison operations
  */
+static int
+float8_cmp_internal(float8 a, float8 b)
+{
+	/*
+	 * We consider all NANs to be equal and larger than any non-NAN. This
+	 * is somewhat arbitrary; the important thing is to have a consistent
+	 * sort order.
+	 */
+	if (isnan(a))
+	{
+		if (isnan(b))
+			return 0;			/* NAN = NAN */
+		else
+			return 1;			/* NAN > non-NAN */
+	}
+	else if (isnan(b))
+	{
+		return -1;				/* non-NAN < NAN */
+	}
+	else
+	{
+		if (a > b)
+			return 1;
+		else if (a < b)
+			return -1;
+		else
+			return 0;
+	}
+}
+
 Datum
 float8eq(PG_FUNCTION_ARGS)
 {
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 == arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) == 0);
 }
 
 Datum
@@ -608,7 +791,7 @@ float8ne(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 != arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) != 0);
 }
 
 Datum
@@ -617,7 +800,7 @@ float8lt(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 < arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) < 0);
 }
 
 Datum
@@ -626,7 +809,7 @@ float8le(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 <= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) <= 0);
 }
 
 Datum
@@ -635,7 +818,7 @@ float8gt(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 > arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) > 0);
 }
 
 Datum
@@ -644,7 +827,16 @@ float8ge(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 >= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) >= 0);
+}
+
+Datum
+btfloat8cmp(PG_FUNCTION_ARGS)
+{
+	float8		arg1 = PG_GETARG_FLOAT8(0);
+	float8		arg2 = PG_GETARG_FLOAT8(1);
+
+	PG_RETURN_INT32(float8_cmp_internal(arg1, arg2));
 }
 
 
@@ -690,7 +882,9 @@ dtoi4(PG_FUNCTION_ARGS)
 	int32		result;
 
 	if ((num < INT_MIN) || (num > INT_MAX))
-		elog(ERROR, "dtoi4: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int32) rint(num);
 	PG_RETURN_INT32(result);
@@ -707,7 +901,9 @@ dtoi2(PG_FUNCTION_ARGS)
 	int16		result;
 
 	if ((num < SHRT_MIN) || (num > SHRT_MAX))
-		elog(ERROR, "dtoi2: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int16) rint(num);
 	PG_RETURN_INT16(result);
@@ -752,7 +948,9 @@ ftoi4(PG_FUNCTION_ARGS)
 	int32		result;
 
 	if ((num < INT_MIN) || (num > INT_MAX))
-		elog(ERROR, "ftoi4: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int32) rint(num);
 	PG_RETURN_INT32(result);
@@ -769,7 +967,9 @@ ftoi2(PG_FUNCTION_ARGS)
 	int16		result;
 
 	if ((num < SHRT_MIN) || (num > SHRT_MAX))
-		elog(ERROR, "ftoi2: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int16) rint(num);
 	PG_RETURN_INT16(result);
@@ -926,6 +1126,50 @@ dround(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(result);
 }
 
+/*
+ *		dceil			- returns the smallest integer greater than or
+ *						  equal to the specified float
+ */
+Datum
+dceil(PG_FUNCTION_ARGS)
+{
+	float8		arg1 = PG_GETARG_FLOAT8(0);
+
+	PG_RETURN_FLOAT8(ceil(arg1));
+}
+
+/*
+ *		dfloor			- returns the largest integer lesser than or
+ *						  equal to the specified float
+ */
+Datum
+dfloor(PG_FUNCTION_ARGS)
+{
+	float8		arg1 = PG_GETARG_FLOAT8(0);
+
+	PG_RETURN_FLOAT8(floor(arg1));
+}
+
+/*
+ *		dsign			- returns -1 if the argument is less than 0, 0
+ *						  if the argument is equal to 0, and 1 if the
+ *						  argument is greater than zero.
+ */
+Datum
+dsign(PG_FUNCTION_ARGS)
+{
+	float8		arg1 = PG_GETARG_FLOAT8(0);
+	float8		result;
+
+	if (arg1 > 0)
+		result = 1.0;
+	else if (arg1 < 0)
+		result = -1.0;
+	else
+		result = 0.0;
+
+	PG_RETURN_FLOAT8(result);
+}
 
 /*
  *		dtrunc			- returns truncation-towards-zero of arg1,
@@ -959,7 +1203,9 @@ dsqrt(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg1 < 0)
-		elog(ERROR, "can't take sqrt of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_FLOATING_POINT_EXCEPTION),
+				 errmsg("cannot take square root of a negative number")));
 
 	result = sqrt(arg1);
 
@@ -1003,7 +1249,9 @@ dpow(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "pow() result is out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("result is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1031,7 +1279,9 @@ dexp(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "exp() result is out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("result is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1049,9 +1299,14 @@ dlog1(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg1 == 0.0)
-		elog(ERROR, "can't take log of zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_FLOATING_POINT_EXCEPTION),
+				 errmsg("cannot take logarithm of zero")));
+
 	if (arg1 < 0)
-		elog(ERROR, "can't take log of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_FLOATING_POINT_EXCEPTION),
+				 errmsg("cannot take logarithm of a negative number")));
 
 	result = log(arg1);
 
@@ -1070,9 +1325,14 @@ dlog10(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg1 == 0.0)
-		elog(ERROR, "can't take log of zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_FLOATING_POINT_EXCEPTION),
+				 errmsg("cannot take logarithm of zero")));
+
 	if (arg1 < 0)
-		elog(ERROR, "can't take log of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_FLOATING_POINT_EXCEPTION),
+				 errmsg("cannot take logarithm of a negative number")));
 
 	result = log10(arg1);
 
@@ -1097,7 +1357,9 @@ dacos(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "acos(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1120,7 +1382,9 @@ dasin(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "asin(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1143,7 +1407,9 @@ datan(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "atan(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1167,7 +1433,9 @@ datan2(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "atan2(%f,%f) input is out of range", arg1, arg2);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1190,7 +1458,9 @@ dcos(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "cos(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1213,7 +1483,9 @@ dcot(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "cot(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	result = 1.0 / result;
 	CheckFloat8Val(result);
@@ -1237,7 +1509,9 @@ dsin(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "sin(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1260,7 +1534,9 @@ dtan(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "tan(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1363,15 +1639,14 @@ setseed(PG_FUNCTION_ARGS)
 static float8 *
 check_float8_array(ArrayType *transarray, const char *caller)
 {
-
 	/*
 	 * We expect the input to be a 3-element float array; verify that. We
 	 * don't need to use deconstruct_array() since the array data is just
 	 * going to look like a C array of 3 float8 values.
 	 */
-	if (ARR_SIZE(transarray) != (ARR_OVERHEAD(1) + 3 * sizeof(float8)) ||
-		ARR_NDIM(transarray) != 1 ||
-		ARR_DIMS(transarray)[0] != 3)
+	if (ARR_NDIM(transarray) != 1 ||
+		ARR_DIMS(transarray)[0] != 3 ||
+		ARR_ELEMTYPE(transarray) != FLOAT8OID)
 		elog(ERROR, "%s: expected 3-element float8 array", caller);
 	return (float8 *) ARR_DATA_PTR(transarray);
 }
@@ -1402,7 +1677,8 @@ float8_accum(PG_FUNCTION_ARGS)
 	transdatums[2] = Float8GetDatumFast(sumX2);
 
 	result = construct_array(transdatums, 3,
-						 false /* float8 byval */ , sizeof(float8), 'd');
+							 FLOAT8OID,
+						 sizeof(float8), false /* float8 byval */ , 'd');
 
 	PG_RETURN_ARRAYTYPE_P(result);
 }
@@ -1437,7 +1713,8 @@ float4_accum(PG_FUNCTION_ARGS)
 	transdatums[2] = Float8GetDatumFast(sumX2);
 
 	result = construct_array(transdatums, 3,
-						 false /* float8 byval */ , sizeof(float8), 'd');
+							 FLOAT8OID,
+						 sizeof(float8), false /* float8 byval */ , 'd');
 
 	PG_RETURN_ARRAYTYPE_P(result);
 }
@@ -1469,21 +1746,25 @@ float8_variance(PG_FUNCTION_ARGS)
 	float8	   *transvalues;
 	float8		N,
 				sumX,
-				sumX2;
+				sumX2,
+				numerator;
 
 	transvalues = check_float8_array(transarray, "float8_variance");
 	N = transvalues[0];
 	sumX = transvalues[1];
 	sumX2 = transvalues[2];
 
-	/* We define VARIANCE of no values to be NULL, of 1 value to be 0 */
-	if (N == 0.0)
+	/* Sample variance is undefined when N is 0 or 1, so return NULL */
+	if (N <= 1.0)
 		PG_RETURN_NULL();
 
-	if (N <= 1.0)
+	numerator = N * sumX2 - sumX * sumX;
+
+	/* Watch out for roundoff error producing a negative numerator */
+	if (numerator <= 0.0)
 		PG_RETURN_FLOAT8(0.0);
 
-	PG_RETURN_FLOAT8((N * sumX2 - sumX * sumX) / (N * (N - 1.0)));
+	PG_RETURN_FLOAT8(numerator / (N * (N - 1.0)));
 }
 
 Datum
@@ -1493,21 +1774,25 @@ float8_stddev(PG_FUNCTION_ARGS)
 	float8	   *transvalues;
 	float8		N,
 				sumX,
-				sumX2;
+				sumX2,
+				numerator;
 
 	transvalues = check_float8_array(transarray, "float8_stddev");
 	N = transvalues[0];
 	sumX = transvalues[1];
 	sumX2 = transvalues[2];
 
-	/* We define STDDEV of no values to be NULL, of 1 value to be 0 */
-	if (N == 0.0)
+	/* Sample stddev is undefined when N is 0 or 1, so return NULL */
+	if (N <= 1.0)
 		PG_RETURN_NULL();
 
-	if (N <= 1.0)
+	numerator = N * sumX2 - sumX * sumX;
+
+	/* Watch out for roundoff error producing a negative numerator */
+	if (numerator <= 0.0)
 		PG_RETURN_FLOAT8(0.0);
 
-	PG_RETURN_FLOAT8(sqrt((N * sumX2 - sumX * sumX) / (N * (N - 1.0))));
+	PG_RETURN_FLOAT8(sqrt(numerator / (N * (N - 1.0))));
 }
 
 
@@ -1567,7 +1852,9 @@ float48div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float48div: divide by zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 	CheckFloat8Val(result);
@@ -1627,7 +1914,9 @@ float84div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float84div: divide by zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 
@@ -1650,7 +1939,7 @@ float48eq(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 == arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) == 0);
 }
 
 Datum
@@ -1659,7 +1948,7 @@ float48ne(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 != arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) != 0);
 }
 
 Datum
@@ -1668,7 +1957,7 @@ float48lt(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 < arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) < 0);
 }
 
 Datum
@@ -1677,7 +1966,7 @@ float48le(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 <= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) <= 0);
 }
 
 Datum
@@ -1686,7 +1975,7 @@ float48gt(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 > arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) > 0);
 }
 
 Datum
@@ -1695,7 +1984,7 @@ float48ge(PG_FUNCTION_ARGS)
 	float4		arg1 = PG_GETARG_FLOAT4(0);
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 
-	PG_RETURN_BOOL(arg1 >= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) >= 0);
 }
 
 /*
@@ -1707,7 +1996,7 @@ float84eq(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 == arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) == 0);
 }
 
 Datum
@@ -1716,7 +2005,7 @@ float84ne(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 != arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) != 0);
 }
 
 Datum
@@ -1725,7 +2014,7 @@ float84lt(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 < arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) < 0);
 }
 
 Datum
@@ -1734,7 +2023,7 @@ float84le(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 <= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) <= 0);
 }
 
 Datum
@@ -1743,7 +2032,7 @@ float84gt(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 > arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) > 0);
 }
 
 Datum
@@ -1752,117 +2041,12 @@ float84ge(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 
-	PG_RETURN_BOOL(arg1 >= arg2);
+	PG_RETURN_BOOL(float8_cmp_internal(arg1, arg2) >= 0);
 }
 
 /* ========== PRIVATE ROUTINES ========== */
 
-/* From "fdlibm" @ netlib.att.com */
-
-#ifndef HAVE_RINT
-
-/* @(#)s_rint.c 5.1 93/09/24 */
-/*
- * ====================================================
- * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
- *
- * Developed at SunPro, a Sun Microsystems, Inc. business.
- * Permission to use, copy, modify, and distribute this
- * software is freely granted, provided that this notice
- * is preserved.
- * ====================================================
- */
-
-/*
- * rint(x)
- * Return x rounded to integral value according to the prevailing
- * rounding mode.
- * Method:
- *		Using floating addition.
- * Exception:
- *		Inexact flag raised if x not equal to rint(x).
- */
-
-static const double one = 1.0,
-			TWO52[2] = {
-	4.50359962737049600000e+15, /* 0x43300000, 0x00000000 */
-	-4.50359962737049600000e+15,/* 0xC3300000, 0x00000000 */
-};
-
-static double
-rint(double x)
-{
-	int			i0,
-				n0,
-				j0,
-				sx;
-	unsigned	i,
-				i1;
-	double		w,
-				t;
-
-	n0 = (*((int *) &one) >> 29) ^ 1;
-	i0 = *(n0 + (int *) &x);
-	sx = (i0 >> 31) & 1;
-	i1 = *(1 - n0 + (int *) &x);
-	j0 = ((i0 >> 20) & 0x7ff) - 0x3ff;
-	if (j0 < 20)
-	{
-		if (j0 < 0)
-		{
-			if (((i0 & 0x7fffffff) | i1) == 0)
-				return x;
-			i1 |= (i0 & 0x0fffff);
-			i0 &= 0xfffe0000;
-			i0 |= ((i1 | -i1) >> 12) & 0x80000;
-			*(n0 + (int *) &x) = i0;
-			w = TWO52[sx] + x;
-			t = w - TWO52[sx];
-			i0 = *(n0 + (int *) &t);
-			*(n0 + (int *) &t) = (i0 & 0x7fffffff) | (sx << 31);
-			return t;
-		}
-		else
-		{
-			i = (0x000fffff) >> j0;
-			if (((i0 & i) | i1) == 0)
-				return x;		/* x is integral */
-			i >>= 1;
-			if (((i0 & i) | i1) != 0)
-			{
-				if (j0 == 19)
-					i1 = 0x40000000;
-				else
-					i0 = (i0 & (~i)) | ((0x20000) >> j0);
-			}
-		}
-	}
-	else if (j0 > 51)
-	{
-		if (j0 == 0x400)
-			return x + x;		/* inf or NaN */
-		else
-			return x;			/* x is integral */
-	}
-	else
-	{
-		i = ((unsigned) (0xffffffff)) >> (j0 - 20);
-		if ((i1 & i) == 0)
-			return x;			/* x is integral */
-		i >>= 1;
-		if ((i1 & i) != 0)
-			i1 = (i1 & (~i)) | ((0x40000000) >> (j0 - 20));
-	}
-	*(n0 + (int *) &x) = i0;
-	*(1 - n0 + (int *) &x) = i1;
-	w = TWO52[sx] + x;
-	return w - TWO52[sx];
-}
-
-#endif	 /* !HAVE_RINT */
-
 #ifndef HAVE_CBRT
-
 static double
 cbrt(double x)
 {
@@ -1872,4 +2056,4 @@ cbrt(double x)
 	return isneg ? -tmpres : tmpres;
 }
 
-#endif	 /* !HAVE_CBRT */
+#endif   /* !HAVE_CBRT */
